@@ -4,16 +4,15 @@ from starlette.datastructures import Headers, MutableHeaders
 from starlette.responses import PlainTextResponse, Response
 from starlette.types import ASGIApp, Message, Receive, Scope, Send
 from fastapi import FastAPI,Depends,Response,HTTPException, Request
-from fastapi.middleware.cors import CORSMiddleware
-from starlette.middleware import Middleware
 from starlette.datastructures import MutableHeaders
 import time
 import json
 from sixth import schemas
 import re
 import requests
-import pickledb
 import ast
+from sixth.middlewares.six_base_http_middleware import SixBaseHTTPMiddleware
+
 
 
 class SixRateLimiterMiddleware(BaseHTTPMiddleware):
@@ -25,7 +24,6 @@ class SixRateLimiterMiddleware(BaseHTTPMiddleware):
         self._apikey = apikey
         self._route_last_updated = {}
         self._rate_limit_logs_sent = {}
-        self.db = pickledb.load(f'{apikey}_db.db', auto_dump=True)
         
         for route in fastapi_app.router.routes:
             if type(route.app) == FastAPI:
@@ -100,61 +98,81 @@ class SixRateLimiterMiddleware(BaseHTTPMiddleware):
         
     async def dispatch(self,request: Request,call_next) -> None:
         host = request.client.host
+        
         route = request.scope["path"]
         route = re.sub(r'\W+', '~', route)
         headers = request.headers
         query_params = request.query_params
         rate_limit_resp = None
         status_code = 200
-
+       
         
         
         #fail safe if there is an internal server error our servers are currenly in maintnance
         try:
             update_time = time.time()
-            if update_time - self._route_last_updated[route] >60:
+            if update_time - self._route_last_updated[route] >10:
                 #update rate limit details every 60 seconds
                 rate_limit_resp = requests.get("https://backend.withsix.co/project-config/config/get-route-rate-limit/"+self._apikey+"/"+route)
                 self._route_last_updated[route] = update_time
                 status_code = rate_limit_resp.status_code
-            body = None
 
+            body = None
+          
             try:
                 body = await request.body()
                 await self.set_body(request, body)
                 body = await self._parse_bools(body)
-            except:
+            except Exception as e:
                 pass
-            
             if status_code == 200: 
                 try:
                     rate_limit = schemas.RateLimiter.model_validate(rate_limit_resp.json()) if rate_limit_resp != None else self._config.rate_limiter[route]
                     if rate_limit.is_active:
                         self._config.rate_limiter[route] = rate_limit
                         preferred_id = self._config.rate_limiter[route].unique_id
+                        rules_object={}
                     
                         if preferred_id == "" or preferred_id=="host":
                             preferred_id = host
                             
+                            
+                            
                         else:
                             if rate_limit.rate_limit_type == "body":
                                 if body != None:
                                     preferred_id = body[preferred_id]
+                                    
                                 else:
-                                    _response = await call_next(request)
-                                    return _response
+                                    pass
                             elif rate_limit.rate_limit_type == "header":
                                 preferred_id = headers[preferred_id]
+                                
                             elif rate_limit.rate_limit_type == "args":
                                 preferred_id = query_params[preferred_id]
+                                
                             else:
                                 preferred_id = host
+                        rules_object["default"]=preferred_id
+                        if rate_limit.rate_limit_by_rules:
+                            for key in rate_limit.rate_limit_by_rules:
+                                if key == "body":
+                                    if body != None:
+                                        rules_object["body"] = body[rate_limit.rate_limit_by_rules["body"]]
+                                    else:
+                                        pass
+                                elif key == "header":
+                                    rules_object["headers"] = headers[rate_limit.rate_limit_by_rules["headers"]]
+                                elif key == "args":
+                                    rules_object["args"] = query_params[rate_limit.rate_limit_by_rules["args"]]
+                                
                         
+                        prev_rate_limit_res = False
+                        final_rule = ""
+                        for key in rules_object:
+                            final_rule += rules_object[key]
 
-                        if not self._is_rate_limit_reached(preferred_id, route): 
-                            _response = await call_next(request)
-                            return _response
-                        else:
+                        if self._is_rate_limit_reached(final_rule, route): 
                             await self._send_logs(route=route, header=headers, body=body, query=query_params)
                             temp_payload = rate_limit.error_payload.values()
                             final = {}
@@ -162,9 +180,15 @@ class SixRateLimiterMiddleware(BaseHTTPMiddleware):
                                 for keys in c:
                                     if keys != "uid":
                                         final[keys] = c[keys]
+                                        
                             output= final
                             headers = MutableHeaders(headers={"content-length": str(len(str(output).encode())), 'content-type': 'application/json'})
                             return Response(json.dumps(output), status_code=420, headers=headers)
+                        else:
+                            pass
+                        _response = await call_next(request)
+                        return _response
+                                
                     else:
                         _response = await call_next(request)
                         return _response
